'use strict';

import path from 'path';
import Knex from 'knex';
import Quture from 'quture-app';
import Config from 'quture-config';

export default class Database {

  /**
   * Create a new {Database} instance.
   *
   * @param {Quture} quture
   * @param {boolean} migrateLatest
   */
  constructor(quture, migrateLatest) {
    const self = this;
    if (quture instanceof Quture) {
      this._quture = quture;
      this._config = new Config({
        configDir: path.join(this._quture.getConfig().getConfigDirectory(), 'database')
      });

      /**
       * Publish configuration file
       */
      Quture.getUtilities().publishFile(path.join(__dirname, 'config', 'connection.js'), path.join(this._config.getConfigDirectory()));

      /**
       * Configure knex
       */
      this._migrationsDir = this._config.get('connection.migrations', path.join(this._quture.getRootDirectory(), 'migrations'));
      
      let options = {
        client: this._config.get('connection.dialect', 'sqlite'),
        connection: {
          host: this._config.get('connection.host', '127.0.0.1'),
          user: this._config.get('connection.username', 'root'),
          password: this._config.get('connection.password', ''),
          database: this._config.get('connection.database', 'quture'),
          filename: this._config.get('connection.storage', path.join(this._quture.getStorageDirectory(), 'database.sqlite'))
        },
        migrations: {
          directory: this._migrationsDir,
          tableName: 'migrations'
        }
      };

      /**
       * Instantiate knex
       */
      this._knex = Knex(options);

      /**
       * Execute migration scripts
       */
      if (!migrateLatest || (typeof migrateLatest === 'boolean' && migrateLatest === true)) {
        this._knex.migrate.latest({
          directory: options.migrations.directory
        });
      } else {
        throw new Error('{migrateLatest} should be true or not specified');
      }

    } else {
      throw new Error('{quture} parameter must be an instance of {Quture}');
    }
  }

  /**
   * Retrieve the {Knex} instance.
   *
   * @returns {Knex}
   */
  knex() {
    return this._knex;
  }

}
